#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <stdlib.h>

#include "utils.h"
#include "lcd.h"

int main()
{
	int i, j, rc_x, rc_y;
	int turn = 1;
	int count = 5;
	char buf[1], c;

	initialize_USART(103);

	/* IO init */
	initialize_buttons();

	/* LCD init */
	initialize_LCD();
	lcd_contrast(0x40);

	/* Control matrix init */
	initialize_map_matrix();

	/* Start game */
	initial_slave_screen();

	/* Place airplane */
	while (1)
	{
		if (!(PINB & (1 << PB4)))
		{
			lcd_clear();
			_delay_ms(20000);

			draw_map_to_lcd();
			draw_current_pos_on_LCD(7 + c_x * 8, c_y + 2);
		}

		if (!(PINB & (1 << PB5)))
		{
			if (c_x >= 2 && c_x <= 6)
			{
				save_ship_coords(7 + c_x * 8, c_y + 2);
				mark_my_battleship(c_x, c_y);
				p1_x = c_x;
				p1_y = c_y;

				_delay_ms(20000);
				break;
			}
		}

		if (!(PINB & (1 << PB0)))
		{
			on_button_press(PB0, turn);
		}

		if (!(PINB & (1 << PB1)))
		{
			on_button_press(PB1, turn);
		}

		if (!(PINB & (1 << PB3)))
		{
			on_button_press(PB3, turn);
		}

		if (!(PINB & (1 << PB2)))
		{
			on_button_press(PB2, turn);
		}

		_delay_ms(2000);
	}

	// _delay_ms(2000);

	/* Recv opponent's plane coords */
	c = receive_character();
	rc_x = c;
	c = receive_character();
	rc_y = c;
	mark_enemy_battleship(rc_x, rc_y);
	p2_x = rc_x;
	p2_y = rc_y;
	light_led();
	_delay_ms(20000);
	turnoff();
	/* Send plane coords to opponent */
	unsigned char x_to_send = c_x;
	send_character(x_to_send);
	unsigned char y_to_send = c_y;
	send_character(y_to_send);
	_delay_ms(2000);
	turn = 0;

	/* Game communication */
	while (1)
	{
		// Hit slave plane si send pe seriala
		if (turn)
		{
			lcd_clear();
			my_turn_prompt();
			_delay_ms(20000);

			lcd_clear();
			draw_map_to_lcd();

			for (i = 0; i < X_SIZE; i++)
			{
				for (j = 0; j < Y_SIZE; j++)
				{
					if (matrix_2[i][j] == HIT)
					{
						draw_hit_cell_to_lcd(7 + i * 8, j + 2);
					}

					if (matrix_2[i][j] == MISS)
					{
						draw_miss_cell_to_lcd(7 + i * 8, j + 2);
					}
				}
			}

			// for (i = 0; i < X_SIZE; i++)
			// {
			// 	for (j = 0; j < Y_SIZE; j++)
			// 	{
			// 		if (matrix_2[i][j] == HIT)
			// 		{
			// 			draw_hit_cell_to_lcd(7 + i * 8, j + 2);
			// 		}

			// 		if (matrix_2[i][j] == MISS)
			// 		{
			// 			draw_miss_cell_to_lcd(7 + i * 8, j + 2);
			// 		}

			// 		if (matrix_2[i][j] == PLANE)
			// 		{
			// 			draw_current_pos_on_LCD(7 + i * 8, j + 2);
			// 		}
			// 	}
			// }

			c_x = CENTER_X;
			c_y = CENTER_Y;
			draw_current_pos_on_LCD(7 + c_x * 8, c_y + 2);

			while (1)
			{
				if (!(PINB & (1 << PB4)))
				{
					if (matrix_2[c_x][c_y] == PLANE)
					{
						draw_hit_cell_to_lcd(7 + c_x * 8, c_y + 2);
						matrix_2[c_x][c_y] = HIT;
						count--;
					}
					else if (matrix_2[c_x][c_y] == HIT || matrix_2[c_x][c_y] == MISS)
					{
						// draw_hit_cell_to_lcd(7 + c_x * 8, c_y + 2);
						continue;
					}
					else
					{
						draw_miss_cell_to_lcd(7 + c_x * 8, c_y + 2);
						matrix_2[c_x][c_y] = MISS;
					}

					if (count == 0)
					{
						lcd_clear();
						win_prompt();

						send_character('l');
						// USART0_print("l");
						// itoa(c_y, buf, 10);
						// USART0_print(buf);

						while (1)
							;
					}

					/* Send hit coords to opponent */
					x_to_send = c_x;
					send_character(x_to_send);
					// itoa(c_y, buf, 10);
					y_to_send = c_y;
					send_character(y_to_send);

					_delay_ms(20000);
					turn = 0;
					break;
				}

				if (!(PINB & (1 << PB0)))
				{
					on_button_press(PB0, turn);
				}

				if (!(PINB & (1 << PB1)))
				{
					on_button_press(PB1, turn);
				}

				if (!(PINB & (1 << PB3)))
				{
					on_button_press(PB3, turn);
				}

				if (!(PINB & (1 << PB2)))
				{
					on_button_press(PB2, turn);
				}

				_delay_ms(2000);
			}
		}
		else
		{
			lcd_clear();
			print_wait();
			_delay_ms(20000);

			lcd_clear();
			draw_map_to_lcd();

			for (i = 0; i < X_SIZE; i++)
			{
				for (j = 0; j < Y_SIZE; j++)
				{
					if (matrix_1[i][j] == HIT)
					{
						draw_hit_cell_to_lcd(7 + i * 8, j + 2);
					}

					if (matrix_1[i][j] == MISS)
					{
						draw_miss_cell_to_lcd(7 + i * 8, j + 2);
					}

					if (matrix_1[i][j] == PLANE)
					{
						draw_current_pos_on_LCD(7 + i * 8, j + 2);
					}
				}
			}

			/* Recv opponent's hit coords */
			c = receive_character();

			if (c == 'l')
			{
				lcd_clear();
				lose_prompt();

				while (1)
					;
			}
			// while (c > '9' && c < '0')
			// {
			// 	light_led();
			// 	_delay_ms(20000);
			// 	turnoff();
			// 	_delay_ms(20000);
			// 	c = receive_character();
			// }

			rc_x = c;
			c = receive_character();
			// while (c > '9' && c < '0')
			// {
			// 	light_led();
			// 	_delay_ms(20000);
			// 	turnoff();
			// 	_delay_ms(20000);
			// 	c = receive_character();
			// }
			rc_y = c;

			// if (rc_x == -1)
			// {
			// 	lcd_clear();
			// 	lose_prompt();

			// 	while (1)
			// 		;
			// }

			if (matrix_1[rc_x][rc_y] == PLANE)
			{
				draw_hit_cell_to_lcd(7 + rc_x * 8, rc_y + 2);
				matrix_1[rc_x][rc_y] = HIT;
			}
			else
			{
				draw_miss_cell_to_lcd(7 + rc_x * 8, rc_y + 2);
				matrix_1[rc_x][rc_y] = MISS;
			}

			_delay_ms(20000);
			turn = 1;
		}

		_delay_ms(2000);
	}

	return 0;
}
