#include <avr/io.h>

#include "utils.h"
#include "lcd.h"

void initialize_USART(unsigned int baud)
{
	/* setează baud rate */
	UBRR0H = (unsigned char)(baud >> 8);
	UBRR0L = (unsigned char)baud;

	/* pornește transmițătorul */
	UCSR0B = (1 << TXEN0) | (1 << RXEN0);

	/**
	 * setează formatul frame-ului: 8 biți de date, 2 biți de stop,
	 * fără paritate
	 */
	UCSR0C = (1 << USBS0) | (3 << UCSZ00);
}

void send_character(unsigned char data)
{
	/* așteaptă până bufferul e gol */
	// while (!(UCSR0A & (1 << UDRE0)))
	// 	;
	loop_until_bit_is_set(UCSR0A, UDRE0);
	/* pune datele în buffer; transmisia va porni automat în urma scrierii */
	UDR0 = data;
}

char receive_character()
{
	/* asteapta cat timp bufferul e gol */
	// while (!(UCSR0A & (1 << RXC0)))
	// 	;
	loop_until_bit_is_set(UCSR0A, RXC0);
	/* returneaza datele din buffer */
	return UDR0;
}

void USART0_print(char *data)
{
	while (*data != '\0')
		send_character(*data++);
}

/* Seteaza butoanele folosite ca input */
void initialize_buttons(void)
{
	DDRD |= (1 << PD7);
	PORTD &= ~(1 << PD7);

	DDRB &= ~(1 << PB0) & ~(1 << PB1) & ~(1 << PB2) & ~(1 << PB3) & ~(1 << PB4) & ~(1 << PB5);
	PORTB |= (1 << PB0) | (1 << PB1) | (1 << PB2) | (1 << PB3) | (1 << PB4) | (1 << PB5);
}

/* Initializeaza matricele in care se stocheaza informatiile HIT/MISS/PLANE */
void initialize_map_matrix(void)
{
	int i, j;

	for (i = 0; i < X_SIZE; i++)
	{
		for (j = 0; j < Y_SIZE; j++)
		{
			matrix_1[i][j] = EMPTY;
			matrix_2[i][j] = EMPTY;
		}
	}

	c_x = CENTER_X;
	c_y = CENTER_Y;
}

/* Stocheaza avionul jucatorului */
void mark_my_battleship(int x, int y)
{
	matrix_1[x][y] = PLANE;
	matrix_1[x + 1][y] = PLANE;
	matrix_1[x + 2][y] = PLANE;
	matrix_1[x - 1][y] = PLANE;
	matrix_1[x - 2][y] = PLANE;

	// matrix_1[x][y - 1] = PLANE;
	// matrix_1[x][y + 1] = PLANE;
	// matrix_1[x][y + 2] = PLANE;

	// matrix_1[x - 1][y + 2] = PLANE;
	// matrix_1[x + 1][y + 2] = PLANE;
}

/* Stocheaza avionul oponentului */
void mark_enemy_battleship(int x, int y)
{
	matrix_2[x][y] = PLANE;
	matrix_2[x + 1][y] = PLANE;
	matrix_2[x + 2][y] = PLANE;
	matrix_2[x - 1][y] = PLANE;
	matrix_2[x - 2][y] = PLANE;

	// matrix_2[x][y - 1] = PLANE;
	// matrix_2[x][y + 1] = PLANE;
	// matrix_2[x][y + 2] = PLANE;

	// matrix_2[x - 1][y + 2] = PLANE;
	// matrix_2[x + 1][y + 2] = PLANE;
}

/* Deplaseaza cursorul */

void light_led()
{
	DDRD |= (1 << PD7);
	PORTD |= (1 << PD7);
}

void turnoff()
{
	DDRD |= (1 << PD7);
	PORTD &= ~(1 << PD7);
}

void on_button_press(int button, int turn)
{
	if (button == PB0)
	{
		if (c_y > 0)
		{
			move_cursor_up(7 + c_x * 8, c_y + 2);

			if (!turn)
			{
				if (matrix_1[c_x][c_y] == PLANE)
				{
					draw_current_pos_on_LCD(7 + c_x * 8, c_y + 2);
				}

				if (matrix_1[c_x][c_y] == HIT)
				{
					draw_hit_cell_to_lcd(7 + c_x * 8, c_y + 2);
				}

				if (matrix_1[c_x][c_y] == MISS)
				{
					draw_miss_cell_to_lcd(7 + c_x * 8, c_y + 2);
				}

				c_y -= 1;
			}
			else
			{
				if (matrix_2[c_x][c_y] == HIT)
				{
					draw_hit_cell_to_lcd(7 + c_x * 8, c_y + 2);
				}

				if (matrix_2[c_x][c_y] == MISS)
				{
					draw_miss_cell_to_lcd(7 + c_x * 8, c_y + 2);
				}

				c_y -= 1;
			}
		}
	}

	if (button == PB1)
	{
		if (c_x > 0)
		{
			move_cursor_left(7 + c_x * 8, c_y + 2);

			if (!turn)
			{
				if (matrix_1[c_x][c_y] == PLANE)
				{
					draw_current_pos_on_LCD(7 + c_x * 8, c_y + 2);
				}

				if (matrix_1[c_x][c_y] == HIT)
				{
					draw_hit_cell_to_lcd(7 + c_x * 8, c_y + 2);
				}

				if (matrix_1[c_x][c_y] == MISS)
				{
					draw_miss_cell_to_lcd(7 + c_x * 8, c_y + 2);
				}

				c_x -= 1;
			}
			else
			{
				if (matrix_2[c_x][c_y] == HIT)
				{
					draw_hit_cell_to_lcd(7 + c_x * 8, c_y + 2);
				}

				if (matrix_2[c_x][c_y] == MISS)
				{
					draw_miss_cell_to_lcd(7 + c_x * 8, c_y + 2);
				}

				c_x -= 1;
			}
		}
	}

	if (button == PB3)
	{
		if (c_x < X_SIZE - 1)
		{
			move_cursor_right(7 + c_x * 8, c_y + 2);

			if (!turn)
			{
				if (matrix_1[c_x][c_y] == PLANE)
				{
					draw_current_pos_on_LCD(7 + c_x * 8, c_y + 2);
				}

				if (matrix_1[c_x][c_y] == HIT)
				{
					draw_hit_cell_to_lcd(7 + c_x * 8, c_y + 2);
				}

				if (matrix_1[c_x][c_y] == MISS)
				{
					draw_miss_cell_to_lcd(7 + c_x * 8, c_y + 2);
				}

				c_x += 1;
			}
			else
			{
				if (matrix_2[c_x][c_y] == HIT)
				{
					draw_hit_cell_to_lcd(7 + c_x * 8, c_y + 2);
				}

				if (matrix_2[c_x][c_y] == MISS)
				{
					draw_miss_cell_to_lcd(7 + c_x * 8, c_y + 2);
				}

				c_x += 1;
			}
		}
	}

	if (button == PB2)
	{
		if (c_y < Y_SIZE - 1)
		{
			move_cursor_down(7 + c_x * 8, c_y + 2);

			if (!turn)
			{
				if (matrix_1[c_x][c_y] == PLANE)
				{
					draw_current_pos_on_LCD(7 + c_x * 8, c_y + 2);
				}

				if (matrix_1[c_x][c_y] == HIT)
				{
					draw_hit_cell_to_lcd(7 + c_x * 8, c_y + 2);
				}

				if (matrix_1[c_x][c_y] == MISS)
				{
					draw_miss_cell_to_lcd(7 + c_x * 8, c_y + 2);
				}

				c_y += 1;
			}
			else
			{
				if (matrix_2[c_x][c_y] == HIT)
				{
					draw_hit_cell_to_lcd(7 + c_x * 8, c_y + 2);
				}

				if (matrix_2[c_x][c_y] == MISS)
				{
					draw_miss_cell_to_lcd(7 + c_x * 8, c_y + 2);
				}

				c_y += 1;
			}
		}
	}
}
